// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package org.apache.doris.nereids.rules.implementation;

import org.apache.doris.nereids.rules.Rule;
import org.apache.doris.nereids.rules.RuleType;
import org.apache.doris.nereids.trees.plans.logical.LogicalDeferMaterializeOlapScan;
import org.apache.doris.nereids.trees.plans.physical.PhysicalDeferMaterializeOlapScan;
import org.apache.doris.nereids.trees.plans.physical.PhysicalOlapScan;

import java.util.Optional;

/**
 * implement defer materialize olap scan from logical to physical
 */
public class LogicalDeferMaterializeOlapScanToPhysicalDeferMaterializeOlapScan extends OneImplementationRuleFactory {
    @Override
    public Rule build() {
        return logicalDeferMaterializeOlapScan().thenApply(ctx -> {
            LogicalDeferMaterializeOlapScan logicalDeferOlapScan = ctx.root;
            PhysicalOlapScan physicalOlapScan = (PhysicalOlapScan) new LogicalOlapScanToPhysicalOlapScan().build()
                    .transform(logicalDeferOlapScan.getLogicalOlapScan(), ctx.cascadesContext).get(0);
            return new PhysicalDeferMaterializeOlapScan(physicalOlapScan,
                    logicalDeferOlapScan.getDeferMaterializeSlotIds(),
                    logicalDeferOlapScan.getColumnIdSlot(),
                    Optional.empty(),
                    logicalDeferOlapScan.getLogicalProperties());
        }).toRule(RuleType.LOGICAL_DEFER_MATERIALIZE_OLAP_SCAN_TO_PHYSICAL_DEFER_MATERIALIZE_OLAP_SCAN_RULE);
    }
}
